<?php
namespace Teknisa\SUP\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use Teknisa\SUP\Services\ApprovalWorkflowService;
use Teknisa\SUP\Controllers\SendEmailController;

class ApprovalWorkflowController
{
    const MENSAGENS = [
        'SEM_APROVADOR' => 'AF aprovada automaticamente devido a falta de aprovador cadastrado.'
    ];
    
    public function __construct(ApprovalWorkflowService $service, SendEmailController $sendEmail)
    {
        $this->service   = $service;
        $this->sendEmail = $sendEmail;
    }
    
    /*************
     * Main Flow *
     *************/
    
    public function checkApprovalWorkflow(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        
        $this->verificaWorkflow($row, $response);
    }
    
    public function insertOrderOnWorkflow($cdFilLanc, $cdFilDest, $nrAutoForn, $operation, $orderType)
    {
        $dadosDemanda = [
            'CDFILLANC'   => $cdFilLanc,
            'CDFILDEST'   => $cdFilDest,
            'NRDEMANDA'   => $nrAutoForn,
            'OPERACAO'    => $operation,
            'IDTPDEMANDA' => $orderType,
        ];
        
        return $this->verificaWorkflow($dadosDemanda);
    }
    
    public function verificaWorkflow($dadosDemanda, $response = null)
    {
        switch ($dadosDemanda['IDTPDEMANDA']) {
            case 'AF':
                if ($dadosDemanda['OPERACAO'] == 'LANCAMENTO') {
                    $resultadoOperacao = $this->operacaoLancamentoAF($dadosDemanda);
                    
                    return $this->getMensagemRetorno($resultadoOperacao, $response);
                }
                
                break;
        }
    }
    
    public function operacaoLancamentoAF($dadosDemanda)
    {
        $dadosAF = $this->findDemanda($dadosDemanda);
        
        $dataEntregaAF = $this->findDataEntregaAF($dadosDemanda);
        
        
        $paramsUpdateDemanda = [
            'CDFILDEST'      => $dadosDemanda['CDFILDEST'],
            'CDFILLANC'      => $dadosDemanda['CDFILLANC'],
            'NRDEMANDA'      => $dadosDemanda['NRDEMANDA'],
            'IDTPDEMANDA'    => $dadosDemanda['IDTPDEMANDA'],
            'IDTIPOLANC'     => $dadosAF['LANCAMENTO'],
            'SITUACAO'       => 'APROVADA',
            'NRCONFIGALCADA' => '',
        ];
        
        $diaInicioOrcamento = $this->findDiaInicioOrcamento($dadosDemanda['CDFILDEST']);
        
        
        $paramsDataInicioOrcamento = [
            'DIA' => $diaInicioOrcamento,
            'DATA' => $dataEntregaAF,
        ];
        
        $dataOrcamento = $this->findDataOrcamento($paramsDataInicioOrcamento);
        
        
        $paramsPeriodoOrcamento = [
            'V_DATAINI' => $dataOrcamento['V_DATAINI'],
            'DIAINIORCA' => $diaInicioOrcamento,
        ];
        
        $periodoOrcamento = $this->findPeriodoOrcamento($paramsPeriodoOrcamento);
        
        
        $paramsExisteOrcamento = [
            'CDFILDEST'        => $dadosDemanda['CDFILDEST'],
            'IDPERIODOORCA'    => $periodoOrcamento['V_IDPERIODOORCA'],
            'NRPERIODOORCAGOP' => $periodoOrcamento['V_NRPERIODOORCAGOP'],
            'DATAINI'          => $dataOrcamento['V_DATAINI'],
            'DATAFIM'          => $dataOrcamento['V_DATAFIM'],
            'DTPROGENTAF'      => $dataEntregaAF,
            'CDFILLANC'        => $dadosDemanda['CDFILLANC'],
            'NRDEMANDA'        => $dadosDemanda['NRDEMANDA'],
            'IDTPDEMANDA'      => $dadosDemanda['IDTPDEMANDA'],
        ];
        
        if (!$this->existeOrcamento($paramsExisteOrcamento)) {
            $this->updateDemanda($paramsUpdateDemanda);
            
            return ['APROVADA'];
        }
        
        if ($this->findQuantidadeFilialAlcadas($dadosDemanda['CDFILDEST']) > 0) {
            $statusOrcamento = $this->service->executeConsisteOrcamentoGOP($dadosDemanda)['P_CONSISTE'];
            
            if ($this->isForaOrcamento($statusOrcamento)) {
                $params = $this->findTipoOrcamento($dadosDemanda);
            } else {
                $params = $this->findDemandaDespesa($dadosDemanda);
                
                if ($params['CDDESPPREV'] == null || $params['CDDESPPREV'] == '') {
                    $params = $this->findDemandaConsumo($dadosDemanda);
                }
            }
            
            $paramsAlcadaLancamento = [
                'CDFILDEST'     => $dadosDemanda['CDFILDEST'],
                'CDFILLANC'     => $dadosDemanda['CDFILLANC'],
                'NRDEMANDA'     => $dadosDemanda['NRDEMANDA'],
                'LANCAMENTO'    => $dadosAF['LANCAMENTO'],
                'IDTIPORCAOP'   => $params['NMITMORCA'],
                'CDDESPPREV'    => $params['CDDESPPREV'],
                'FORAORCAMENTO' => $statusOrcamento == 'S' ? 'N' : 'S',
            ];
            
            $alcada = $this->findAlcadaLancamento($paramsAlcadaLancamento);
            
            if (empty($alcada)) {
                $paramsUpdateDemanda['SITUACAO']       = 'APROVADA';
                $paramsUpdateDemanda['NRCONFIGALCADA'] = null;
                
                $this->updateDemanda($paramsUpdateDemanda);
                
                return ['APROVADA'];
            }
            
            $paramsFindAprovador = [
                'NRCONFIGALCADA' => $alcada['NRCONFIGALCADA'],
                'VRAPROVCMP'     => null,
                'CDFILIAL'       => $dadosDemanda['CDFILDEST'],
            ];
            
            $aprovador = $this->findPrimeiroAprovador($paramsFindAprovador);
            
            if (empty($aprovador)) {
                $paramsUpdateDemanda['SITUACAO']       = 'APROVADA';
                $paramsUpdateDemanda['NRCONFIGALCADA'] = null;
                
                $this->updateDemanda($paramsUpdateDemanda);
                
                return ['APROVADA', 'SEM_APROVADOR'];
            }
            
            $paramsUpdateDemanda['SITUACAO']       = 'PENDENTE';
            $paramsUpdateDemanda['NRCONFIGALCADA'] = $alcada['NRCONFIGALCADA'];
            
            $this->updateDemanda($paramsUpdateDemanda);
            
            $paramsTotalDia = [
                'CDFILDEST'   => $dadosDemanda['CDFILDEST'],
                'CDFILLANC'   => $dadosDemanda['CDFILLANC'],
                'NRDEMANDA'   => $dadosDemanda['NRDEMANDA'],
                'IDTPDEMANDA' => $dadosDemanda['IDTPDEMANDA'],
            ];
            
            $totalDia  = $this->findTotalDia($paramsTotalDia);
    
            $total     = $totalDia + $dadosAF['VRTOTALAF'];
            
            if ($total <= $aprovador['VRAPROVCMP']) {
                $paramsUpdateDemanda['SITUACAO']       = 'APROVADA';
                $paramsUpdateDemanda['NRCONFIGALCADA'] = $alcada['NRCONFIGALCADA'];
            
                $this->updateDemanda($paramsUpdateDemanda);
                
                return ['APROVADA'];
            }
            
            if ($this->isOperadorLancamentoPrimeiroAprovador($dadosAF['CDOPERLANCAF'], $dadosDemanda['CDFILDEST'], $alcada['NRCONFIGALCADA'], $aprovador)) {
                $situacao = 'APROVADA';
            } else {
                $situacao = 'PENDENTE';
            }
            
            $paramsGerarAprovacao = [
                'CDFILDEST'      => $dadosDemanda['CDFILDEST'],
                'CDFILLANC'      => $dadosDemanda['CDFILLANC'],
                'NRCONFIGALCADA' => $alcada['NRCONFIGALCADA'],
                'NRDEMANDA'      => $dadosDemanda['NRDEMANDA'],
                'CDOPERADOR'     => $aprovador['CDOPERADOR'],
                'IDTPDEMANDA'    => $dadosDemanda['IDTPDEMANDA'],
                'IDCLASSIFIC'    => $statusOrcamento,
                'IDOPERACAOALC'  => $dadosDemanda['OPERACAO'],
                'IDSITUACAO'     => $situacao,
                'IDAUTOFORNEX'   => $dadosAF['IDAUTOFORNEX'],
                'APROVADOR'      => $aprovador,
                'CDOPERLANCAF'   => $dadosAF['CDOPERLANCAF'],
            ];
            
            $this->gerarAprovacao($paramsGerarAprovacao);
            
            return ['PENDENTE'];
            
        }
        
        $this->updateDemanda($paramsUpdateDemanda);
        
        return ['APROVADA'];
    }
    
    public function updateDemanda($params)
    {
        if ($params['IDTPDEMANDA'] == 'AF') {
            $this->service->updateDemandaAF($params);
        }
    }
    
    public function gerarAprovacao($params)
    {
        if ($params['IDCLASSIFIC'] == 'S') {
            $params['IDCLASSIFIC'] = 'FORA_ORCAMENTO';
        } else {
            $params['IDCLASSIFIC'] = 'PADRAO';
            
            if ($params['IDAUTOFORNEX'] == 'EXTRA') {
                $params['IDCLASSIFIC'] = 'EXTRA';
            }
        }
        
        $paramsVerificaRegistroAprov = [
            'NRCONFIGALCADA' => $params['NRCONFIGALCADA'],
            'CDFILDEST'      => $params['CDFILDEST'],
            'CDFILLANC'      => $params['CDFILLANC'],
            'NRDEMANDA'      => $params['NRDEMANDA'],
            'CDOPERADOR'     => $params['CDOPERADOR'],
        ];
        
        $registroAprov = $this->service->verificaRegistroAprov($paramsVerificaRegistroAprov);
        
        if (count($registroAprov)) {
            $paramsAtualizarRegistroAprov = [
                'NRCONFIGALCADA' => $params['NRCONFIGALCADA'],
                'CDFILDEST'      => $params['CDFILDEST'],
                'CDFILLANC'      => $params['CDFILLANC'],
                'NRDEMANDA'      => $params['NRDEMANDA'],
                'CDOPERADOR'     => $params['CDOPERADOR'],
                'IDAPROVADO'     => $params['SITUACAO'],
            ];
            
            $this->service->atualizarRegistroAprov($paramsAtualizarRegistroAprov);
        } else {
            $paramsInserirRegistroAprov = [
                'NRCONFIGALCADA' => $params['NRCONFIGALCADA'],
                'CDFILDEST'      => $params['CDFILDEST'],
                'CDFILLANC'      => $params['CDFILLANC'],
                'NRDEMANDA'      => $params['NRDEMANDA'],
                'CDOPERADOR'     => $params['CDOPERADOR'],
                'IDTPDEMANDA'    => $params['IDTPDEMANDA'],
                'IDCLASSIFIC'    => $params['IDCLASSIFIC'],
                'IDOPERACAOALC'  => $params['IDOPERACAOALC'],
                'SITUACAO'       => $params['SITUACAO'],
            ];
            
            $this->service->inserirRegistroAprov($paramsInserirRegistroAprov);
        }
        
        $operadorLanc = $this->findOperadorLancamento($params['CDOPERLANCAF']);
        $emails = $this->service->getOperAddress($params['APROVADOR']['NRALCADAGERFILI']);
        
        $paramsSendMail = [
            'CDFILDEST'     => $params['CDFILDEST'],
            'NRCONFIGALCADA'=> $params['NRCONFIGALCADA'],
            'IDTPDEMANDA'   => $params['IDTPDEMANDA'],
            'NRDEMANDA'     => $params['NRDEMANDA'],
            'IDCLASSIFIC'   => $params['IDCLASSIFIC'],
            'OPERADORLANC'  => $operadorLanc,
            'EMAILS'        => $emails,
        ];
        
        $this->enviarEmail($paramsSendMail);
    }

    public function enviarEmail($params)
    {
        foreach ($params['EMAILS'] as $operAddress) {
            $paramsSendMail = [
                "MOTIVO"           => 'APROVACAO_ALCADA',
                "NMOPERADOR_FROM"  => $params['OPERADORLANC']['NMOPERADOR'],
                "DSEMAILOPER_FROM" => $params['OPERADORLANC']['DSEMAILOPER'],
                "NMOPERADOR"       => $operAddress['NMOPERADOR'],
                "DSEMAILOPER"      => $operAddress['DSEMAILOPER'],
                "IDTPDEMANDA"      => $params['IDTPDEMANDA'],
                "NRAUTOFORN"       => $params['NRDEMANDA'],
                "NMFILIAL"         => $this->findNomeFilial($params['CDFILDEST']),
                "NMALCADA"         => $this->findNomeAlcada($params['NRCONFIGALCADA']),
                "IDCLASSIFIC"      => $params['IDCLASSIFIC'],
            ];
            
            $this->sendEmail->sendMail($paramsSendMail);
        }
        
    }
    
    /**************
     * Validações *
     **************/
    
    public function existeOrcamento($params)
    {
        if ($params['IDTPDEMANDA'] == 'AF') {
            return !empty($this->service->getOrcamentoAF($params));
        }
        
    }
    
    public function isForaOrcamento($statusOrcamento)
    {
        return $statusOrcamento == 'N';
    }
    
    public function isOperadorLancamentoPrimeiroAprovador($cdOperLancAF, $cdFilialDestino, $nrConfigAlcada, $aprovador)
    {
        $paramsValidaOperador = [
            'CDOPERLANC'     => $dadosAF['CDOPERLANCAF'],
            'CDFILDEST'      => $dadosDemanda['CDFILDEST'],
            'NRCONFIGALCADA' => $alcada['NRCONFIGALCADA'],
        ];
        
        $operadorLanc = $this->service->validaOperador($paramsValidaOperador);
        
        if ($operadorLanc && $operadorLanc['CDOPERADOR'] == $aprovador['CDOPERADOR']) {
            return true;
        }
        
        return false;
    }
    
    public function utilizaWorkflow($cdFilial)
    {
        $idUtilWorkflow = $this->service->getIdUtilWorkflow($cdFilial)[0];
        
        if (isset($idUtilWorkflow['IDUTLWORKFLOW'])) {
            return $idUtilWorkflow['IDUTLWORKFLOW'];
        }
        
        return 'N';
    }
    
    /*************
     * Utilities *
     *************/
    
    public function getMensagemRetorno($resultadoOperacao, &$response = null)
    {
        if ($response) {
            foreach ($resultadoOperacao as $statusWorkflow) {
                $mensagem = array_key_exists($statusWorkflow, $this->MENSAGENS) ? $this->MENSAGENS[$statusWorkflow] : $statusWorkflow;
                
                $response->addMessage(new Message($mensagem));
            }
        } else {
            return $resultadoOperacao[0];
        }
    }
    
    /*********
     * Finds *
     *********/
    public function findDemanda($params)
    {
        if ($params['IDTPDEMANDA'] == 'AF') {
            return $this->service->findDemandaAF($params)[0];
        }
    }
    
    public function findTotalDia($params)
    {
        if ($params['IDTPDEMANDA'] == 'AF') {
            return $this->service->findTotalDiaAF($params)[0]['TOTAL'];
        }
    }
    
    public function findQuantidadeFilialAlcadas($cdFilialDestino)
    {
        return $this->service->verificaAlcada($cdFilialDestino)[0]['FILALCADAS'];
    }
    
    public function findAlcadaLancamento($params)
    {
        $alcada = $this->service->findAlcadaLancamento($params);
        
        if ($alcada) {
            return $alcada[0];
        }
    }
    
    public function findPrimeiroAprovador($params)
    {
        $aprovador = $this->service->findPrimeiroAprovador($params);
        
        if ($aprovador) {
            return $aprovador[0];
        }
    }
    
    public function findDiaInicioOrcamento($cdFilialDestino)
    {
        $dados = $this->service->findDiaInicioOrcamento($cdFilialDestino)[0];
        
        return $dados['DIAINIORCA'];
    }
    
    public function findDataOrcamento($params)
    {
        return $this->service->findDataOrcamento($params)[0];
    }
    
    public function findPeriodoOrcamento($params)
    {
        return $this->service->findPeriodoOrcamento($params)[0];
    }
    
    public function findDataEntregaAF($params)
    {
        return $this->service->findDataEntregaAF($params)[0]['DTPROGENTAF'];
    }
    
    public function findOperadorLancamento($params)
    {
        $operadorLanc = $this->service->findOperadorLancamento($params['CDOPERLANCAF']);
        
        if (empty($operadorLanc)) {
            return array (
                'NMOPERADOR'  => '',
                'DSEMAILOPER' => '',
            );
        }
        
        return $operadorLanc[0];
    }
    
    public function findNomeFilial($cdFilialDestino)
    {
        return $this->service->findNomeFilial($cdFilialDestino)[0]['NMFILIAL'];
    }
    
    public function findNomeAlcada($nrConfigAlcada)
    {
        return $this->service->findNomeAlcada($nrConfigAlcada)[0]['NMALCADA'];
    }
    
    public function findTipoOrcamento($params)
    {
        return $this->service->findTipoOrcamento($params)[0];
    }
    
    public function findDemandaDespesa($params)
    {
        return $this->service->findDemandaDespesa($params)[0];
    }
    
    public function findDemandaConsumo($params)
    {
        return $this->service->findDemandaConsumo($params)[0];
    }
}
